/* ick.h -- compilation types for intercal parser */

/* Comment this out if your version of lex automatically supplies yylineno. */
#define YYLINENO_BY_HAND

/* Comment this out if your version of lex doesn't use yyrestart(). */
/*
 * wwp: my flex doesn't use yyrestart(). but it needs yyrestart().
 * see lexer.l for my yyrestart() definition.
 */
#define USE_YYRESTART

typedef INT32	bool;
#define TRUE	1L
#define FALSE	0L

/*
 * We choose this value for maximum number of possible source lines on
 * the theory that no human mind could grok more lines of INTERCAL than
 * this and still retain any vestige of sanity.
 */
#define MAXLINES	1000L

/*
 * Similarly for maximum number of variables
 */
#define MAXVARS		100L

/*
 * Maximum supported statement types; should be equal to (COME_FROM - GETS + 1)
 */
#define MAXTYPES	17L

typedef struct node_t
{
    INT32			opcode;		/* operator or type code */
    unsigned long	constant;	/* constant data attached */
    INT32			width;		/* is this 32-bit data? */
    struct node_t	*lval, *rval;	/* attached expression nodes */
} NODE;

typedef struct tuple_t
{
    UINT32	label;		/* label # of this statement */
    UINT32	comefrom;	/* if NZ, a COME FROM touches this */
    INT32         	exechance;	/* chance of execution */
    UINT32	type;		/* statement type */
    union
    {
	UINT32	target;		/* for NEXT statements */
	NODE		*node;		/* attached expression node(s) */
    } u;
    INT32 lineno; 		/* source line for error messages */
    bool sharedline;    /* if NZ, two statements on a line */
} TUPLE;

/* this maps the `external' name of a variable to an internal array index */
typedef struct
{
    INT32	type;
    INT32 extindex;
    INT32 intindex;
}
atom;

typedef struct
{
    INT32	value;
    char *name;
}
assoc;

extern atom oblist[MAXVARS], *obdex;
extern INT32 nonespots, ntwospots, ntails, nhybrids;

extern TUPLE tuples[MAXLINES];

extern char *enablers[MAXTYPES];
extern assoc vartypes[];

/* the lexical analyzer keeps copies of the source logical lines */
extern char *textlines[MAXLINES];
extern INT32 yylineno;

/* compilation options */
extern bool compile_only; /* just compile into C, don't run the linker */
extern bool nocompilerbug; /* set possibility of E774 to zero */
extern INT32 traditional;   /* compile as INTERCAL-72 */
extern int yydebug;       /* print debugging information while parsing */

extern INT32 politesse;

/* ick.h ends here */
